/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
  *
 * XRT/gauge widget UIM/X integration file
 *
 *  SCCSID -- @(#)XrtGauge.cl.c	1.2    99/12/08 KL Group Inc.
 */

/*****************************************************************************
				XrtGauge.cl.c
*****************************************************************************/

#include <Xm/Xm.h>				/* Motif header file */
#include <Xm/XrtGaugeAll.h>		/* Widget-class public header files */
#include <UxLib.h>

#ifdef	XtName
#undef	XtName
#endif

#include "veos.h"				/* Visual Edge object system */
#include "valuesOf.h"			/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"
#include "UxComp.h"
#include "uxproto.h"

#include "swidget.h"
#include "plist.h"


#include <XrtGauge.cl.h>		/* private swidget-class header file */

#include "XrtGauge_xtype.h"

/* Global variables for the XrtGauge class and its resource descriptors */
Class_t	UxC_XrtGaCircularDial	= NULL_CLASS;
Class_t	UxC_XrtGaCircularGauge	= NULL_CLASS;
Class_t	UxC_XrtGaCounter		= NULL_CLASS;
Class_t	UxC_XrtGaLEDIcon		= NULL_CLASS;
Class_t	UxC_XrtGaLEDNumerical	= NULL_CLASS;
Class_t	UxC_XrtGaLinearDial		= NULL_CLASS;
Class_t	UxC_XrtGaLinearGauge	= NULL_CLASS;
Class_t	UxC_XrtGaPercent		= NULL_CLASS;

Class_t	UxC_XrtGaCenter			= NULL_CLASS;
Class_t	UxC_XrtGaCircularScale	= NULL_CLASS;
Class_t	UxC_XrtGaFrame			= NULL_CLASS;
Class_t	UxC_XrtGaIcon			= NULL_CLASS;
Class_t	UxC_XrtGaIndicator		= NULL_CLASS;
Class_t	UxC_XrtGaItem			= NULL_CLASS;
Class_t	UxC_XrtGaLinearScale	= NULL_CLASS;
Class_t	UxC_XrtGaMarker			= NULL_CLASS;
Class_t	UxC_XrtGaNeedle			= NULL_CLASS;
Class_t	UxC_XrtGaRange			= NULL_CLASS;
Class_t	UxC_XrtGaState			= NULL_CLASS;
Class_t	UxC_XrtGaSwitch			= NULL_CLASS;
Class_t	UxC_XrtGaText			= NULL_CLASS;
Class_t	UxC_XrtGaTick			= NULL_CLASS;


static void presetGadgetResourcesOnXtObject();
static char* XrtGaCircularDialCanAddChild();
static char* XrtGaCircularGaugeCanAddChild();
static char* XrtGaLEDIconCanAddChild();
static char* XrtGaLinearDialCanAddChild();
static char* XrtGaLinearGaugeCanAddChild();
static char* XrtGaPercentCanAddChild();
static void createUnmanaged();

XrtGauge_define_resource(n, table, class)
int 				n;
XrtGaugeResourceDesc	*table;
Class_t				class;
{
	binptr				prpty;
	Resource_t			*pResource;
	XrtGaugeResourceDesc	*pResDesc;
	Division_t			division;
	Pass_t				pass;

	pResDesc = &(table[n]);
	prpty = UxFixed_class_prop(pResDesc->rd_name,
							   class,
							   T_PNTR,
							   pResDesc->offset);
	
	division = (Division_t) pResDesc->division;
	pass = (Pass_t) pResDesc->pass;

	switch(pResDesc->type->datatype.type_gauge) {
	case XRTGA_CALLBACK:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	case XRTGA_DOUBLE:
		/* Resources are accessed through put and get functions */
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_GETFUNCTION,	pResDesc->type->get_func,
							 RD_PUTFUNCTION,	pResDesc->type->put_func,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	default:
			pResource =
				UxDefineResource(RD_NAME,			pResDesc->name,
								 RD_XTNAME,			pResDesc->xt_name,
								 RD_UTYPE,			pResDesc->type->utype,
								 RD_XTYPE,			pResDesc->type->xtype,
								 RD_VALUESOF,		pResDesc->type->values_of,
								 RD_VALIDATOR,		pResDesc->type->validate,
								 RD_EDITFUNCTION,	pResDesc->type->edit_func,
								 RD_READONLY,		pResDesc->readonly,
								 RD_DIVISION,		division,
								 RD_PASS,			pass,
								 RD_END);
	break;
	}

	UxPutClassResource(class, prpty, pResource);
	pResDesc->prpty = prpty;
}

void
UxRegister_XrtGaCenter()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaCenter");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaCenter = UxRegister_class("XrtGaCenter",
									 UxC_gadget,
									 sizeof(UxXrtGaCenter),
									 sizeof(UxXrtGaCenterClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaCenter, "UxXrtGaCenter.h" );
	UxPutToolKitClass(UxC_XrtGaCenter, (char *) xmXrtGaCenterObjectClass);
	UxPutIconBitmap(UxC_XrtGaCenter, "xrtGaCenter.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaCenter, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaCenter);

	presetGadgetResourcesOnXtObject(UxC_XrtGaCenter);

	for (n = 0; n < numXrtGaCenter_res; n++) {
		if (XrtGaCenter_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaCenter_res_table, UxC_XrtGaCenter);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaCenter");
}

void
UxRegister_XrtGaCircularScale()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaCircularScale");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaCircularScale = UxRegister_class("XrtGaCircularScale",
									 UxC_gadget,
									 sizeof(UxXrtGaCircularScale),
									 sizeof(UxXrtGaCircularScaleClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaCircularScale, "UxXrtGaCircularScale.h" );
	UxPutToolKitClass(UxC_XrtGaCircularScale, (char *) xmXrtGaCircularScaleObjectClass);
	UxPutIconBitmap(UxC_XrtGaCircularScale, "xrtGaCircularScale.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaCircularScale, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaCircularScale);

	presetGadgetResourcesOnXtObject(UxC_XrtGaCircularScale);

	for (n = 0; n < numXrtGaCircularScale_res; n++) {
		if (XrtGaCircularScale_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaCircularScale_res_table, UxC_XrtGaCircularScale);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaCircularScale");
}

void
UxRegister_XrtGaFrame()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaFrame");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaFrame = UxRegister_class("XrtGaFrame",
									 UxC_gadget,
									 sizeof(UxXrtGaFrame),
									 sizeof(UxXrtGaFrameClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaFrame, "UxXrtGaFrame.h" );
	UxPutToolKitClass(UxC_XrtGaFrame, (char *) xmXrtGaFrameObjectClass);
	UxPutIconBitmap(UxC_XrtGaFrame, "xrtGaFrame.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaFrame, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaFrame);

	presetGadgetResourcesOnXtObject(UxC_XrtGaFrame);

	for (n = 0; n < numXrtGaFrame_res; n++) {
		if (XrtGaFrame_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaFrame_res_table, UxC_XrtGaFrame);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaFrame");
}

void
UxRegister_XrtGaIcon()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaIcon");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaIcon = UxRegister_class("XrtGaIcon",
									 UxC_gadget,
									 sizeof(UxXrtGaIcon),
									 sizeof(UxXrtGaIconClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaIcon, "UxXrtGaIcon.h" );
	UxPutToolKitClass(UxC_XrtGaIcon, (char *) xmXrtGaIconObjectClass);
	UxPutIconBitmap(UxC_XrtGaIcon, "xrtGaIcon.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaIcon, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaIcon);

	presetGadgetResourcesOnXtObject(UxC_XrtGaIcon);

	for (n = 0; n < numXrtGaIcon_res; n++) {
		if (XrtGaIcon_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaIcon_res_table, UxC_XrtGaIcon);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaIcon");
}

void
UxRegister_XrtGaIndicator()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaIndicator");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaIndicator = UxRegister_class("XrtGaIndicator",
									 UxC_gadget,
									 sizeof(UxXrtGaIndicator),
									 sizeof(UxXrtGaIndicatorClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaIndicator, "UxXrtGaIndicator.h" );
	UxPutToolKitClass(UxC_XrtGaIndicator, (char *) xmXrtGaIndicatorObjectClass);
	UxPutIconBitmap(UxC_XrtGaIndicator, "xrtGaIndicator.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaIndicator, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaIndicator);

	presetGadgetResourcesOnXtObject(UxC_XrtGaIndicator);

	for (n = 0; n < numXrtGaIndicator_res; n++) {
		if (XrtGaIndicator_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaIndicator_res_table, UxC_XrtGaIndicator);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaIndicator");
}

void
UxRegister_XrtGaItem()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaItem");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaItem = UxRegister_class("XrtGaItem",
									 UxC_gadget,
									 sizeof(UxXrtGaItem),
									 sizeof(UxXrtGaItemClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaItem, "UxXrtGaItem.h" );
	UxPutToolKitClass(UxC_XrtGaItem, (char *) xmXrtGaItemObjectClass);
	UxPutIconBitmap(UxC_XrtGaItem, "xrtGaItem.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaItem, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaItem);

	presetGadgetResourcesOnXtObject(UxC_XrtGaItem);

	for (n = 0; n < numXrtGaItem_res; n++) {
		if (XrtGaItem_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaItem_res_table, UxC_XrtGaItem);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaItem");
}

void
UxRegister_XrtGaLinearScale()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaLinearScale");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaLinearScale = UxRegister_class("XrtGaLinearScale",
									 UxC_gadget,
									 sizeof(UxXrtGaLinearScale),
									 sizeof(UxXrtGaLinearScaleClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaLinearScale, "UxXrtGaLinearScale.h" );
	UxPutToolKitClass(UxC_XrtGaLinearScale, (char *) xmXrtGaLinearScaleObjectClass);
	UxPutIconBitmap(UxC_XrtGaLinearScale, "xrtGaLinearScale.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaLinearScale, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaLinearScale);

	presetGadgetResourcesOnXtObject(UxC_XrtGaLinearScale);

	for (n = 0; n < numXrtGaLinearScale_res; n++) {
		if (XrtGaLinearScale_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaLinearScale_res_table, UxC_XrtGaLinearScale);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaLinearScale");
}

void
UxRegister_XrtGaMarker()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaMarker");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaMarker = UxRegister_class("XrtGaMarker",
									 UxC_gadget,
									 sizeof(UxXrtGaMarker),
									 sizeof(UxXrtGaMarkerClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaMarker, "UxXrtGaMarker.h" );
	UxPutToolKitClass(UxC_XrtGaMarker, (char *) xmXrtGaMarkerObjectClass);
	UxPutIconBitmap(UxC_XrtGaMarker, "xrtGaMarker.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaMarker, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaMarker);

	presetGadgetResourcesOnXtObject(UxC_XrtGaMarker);

	for (n = 0; n < numXrtGaMarker_res; n++) {
		if (XrtGaMarker_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaMarker_res_table, UxC_XrtGaMarker);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaMarker");
}

void
UxRegister_XrtGaNeedle()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaNeedle");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaNeedle = UxRegister_class("XrtGaNeedle",
									 UxC_gadget,
									 sizeof(UxXrtGaNeedle),
									 sizeof(UxXrtGaNeedleClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaNeedle, "UxXrtGaNeedle.h" );
	UxPutToolKitClass(UxC_XrtGaNeedle, (char *) xmXrtGaNeedleObjectClass);
	UxPutIconBitmap(UxC_XrtGaNeedle, "xrtGaNeedle.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaNeedle, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaNeedle);

	presetGadgetResourcesOnXtObject(UxC_XrtGaNeedle);

	for (n = 0; n < numXrtGaNeedle_res; n++) {
		if (XrtGaNeedle_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaNeedle_res_table, UxC_XrtGaNeedle);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaNeedle");
}

void
UxRegister_XrtGaRange()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaRange");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaRange = UxRegister_class("XrtGaRange",
									 UxC_gadget,
									 sizeof(UxXrtGaRange),
									 sizeof(UxXrtGaRangeClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaRange, "UxXrtGaRange.h" );
	UxPutToolKitClass(UxC_XrtGaRange, (char *) xmXrtGaRangeObjectClass);
	UxPutIconBitmap(UxC_XrtGaRange, "xrtGaRange.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaRange, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaRange);

	presetGadgetResourcesOnXtObject(UxC_XrtGaRange);

	for (n = 0; n < numXrtGaRange_res; n++) {
		if (XrtGaRange_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaRange_res_table, UxC_XrtGaRange);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaRange");
}

void
UxRegister_XrtGaState()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaState");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaState = UxRegister_class("XrtGaState",
									 UxC_gadget,
									 sizeof(UxXrtGaState),
									 sizeof(UxXrtGaStateClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaState, "UxXrtGaState.h" );
	UxPutToolKitClass(UxC_XrtGaState, (char *) xmXrtGaStateObjectClass);
	UxPutIconBitmap(UxC_XrtGaState, "xrtGaState.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaState, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaState);

	presetGadgetResourcesOnXtObject(UxC_XrtGaState);

	for (n = 0; n < numXrtGaState_res; n++) {
		if (XrtGaState_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaState_res_table, UxC_XrtGaState);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaState");
}

void
UxRegister_XrtGaSwitch()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaSwitch");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaSwitch = UxRegister_class("XrtGaSwitch",
									 UxC_gadget,
									 sizeof(UxXrtGaSwitch),
									 sizeof(UxXrtGaSwitchClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaSwitch, "UxXrtGaSwitch.h" );
	UxPutToolKitClass(UxC_XrtGaSwitch, (char *) xmXrtGaSwitchObjectClass);
	UxPutIconBitmap(UxC_XrtGaSwitch, "xrtGaSwitch.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaSwitch, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaSwitch);

	presetGadgetResourcesOnXtObject(UxC_XrtGaSwitch);

	for (n = 0; n < numXrtGaSwitch_res; n++) {
		if (XrtGaSwitch_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaSwitch_res_table, UxC_XrtGaSwitch);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaSwitch");
}

void
UxRegister_XrtGaText()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaText");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaText = UxRegister_class("XrtGaText",
									 UxC_gadget,
									 sizeof(UxXrtGaText),
									 sizeof(UxXrtGaTextClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaText, "UxXrtGaText.h" );
	UxPutToolKitClass(UxC_XrtGaText, (char *) xmXrtGaTextObjectClass);
	UxPutIconBitmap(UxC_XrtGaText, "xrtGaText.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaText, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaText);

	presetGadgetResourcesOnXtObject(UxC_XrtGaText);

	for (n = 0; n < numXrtGaText_res; n++) {
		if (XrtGaText_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaText_res_table, UxC_XrtGaText);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaText");
}

void
UxRegister_XrtGaTick()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaTick");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaTick = UxRegister_class("XrtGaTick",
									 UxC_gadget,
									 sizeof(UxXrtGaTick),
									 sizeof(UxXrtGaTickClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaTick, "UxXrtGaTick.h" );
	UxPutToolKitClass(UxC_XrtGaTick, (char *) xmXrtGaTickObjectClass);
	UxPutIconBitmap(UxC_XrtGaTick, "xrtGaTick.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaTick, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaTick);

	presetGadgetResourcesOnXtObject(UxC_XrtGaTick);

	for (n = 0; n < numXrtGaTick_res; n++) {
		if (XrtGaTick_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaTick_res_table, UxC_XrtGaTick);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaTick");
}

static char* XrtGaCircularDialCanAddChild (parent, numChildren, childClasses, children)
swidget parent;
int numChildren;
Class_t *childClasses;
swidget *children;
{
	int	 	i;
	
	if (!UxIsSubclass(parent, UxC_XrtGaCircularDial)) {
		return("Parent must be an XrtGaCircularDial widget");
	}

	for (i = 0; i < numChildren; i++) {
		if (childClasses[i] != UxC_XrtGaCircularScale && 
			childClasses[i] != UxC_XrtGaCounter &&
			childClasses[i] != UxC_XrtGaFrame &&
			childClasses[i] != UxC_XrtGaMarker &&
			childClasses[i] != UxC_XrtGaSwitch &&
			childClasses[i] != UxC_XrtGaRange &&
			childClasses[i] != UxC_XrtGaState &&
			childClasses[i] != UxC_XrtGaText &&
			childClasses[i] != UxC_XrtGaTick ) {
			return("XrtGaCircularDial can't accept this object as a child\n");
		}
	}
	return(NULL);
}

/* Function that defines the XrtGaCircularDial class */
void
UxRegister_XrtGaCircularDial()
{
	int		n;
	
	xrt_debug_enter("UxRegister_XrtGaCircularDial");

	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_XrtGaCircularDial = UxRegister_class("XrtGaCircularDial",
									UxC_primitive,
									sizeof(UxXrtGaCircularDial),
									sizeof(UxXrtGaCircularDialClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtGaCircularDial, "UxXrtGaCircularDial.h");
	UxPutToolKitClass(UxC_XrtGaCircularDial, (char *) xmXrtGaCircularDialWidgetClass);
	UxPutIconBitmap(UxC_XrtGaCircularDial, "xrtGaCircularDial.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtGaCircularDial );
	
	for (n = 0; n < numXrtGaCircularDial_res; n++) {
		if (XrtGaCircularDial_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaCircularDial_res_table, UxC_XrtGaCircularDial);
		}
	}
	
	UxInit_method(UxC_XrtGaCircularDial, UxM_UxWidgetCannotAcceptChildren, XrtGaCircularDialCanAddChild);

	xrt_debug_leave("UxRegister_XrtGaCircularDial");
}

static char* XrtGaCircularGaugeCanAddChild (parent, numChildren, childClasses, children)
swidget parent;
int numChildren;
Class_t *childClasses;
swidget *children;
{
	int	 	i;
	
	if (!UxIsSubclass(parent, UxC_XrtGaCircularGauge)) {
		return("Parent must be an XrtGaCircularGauge widget");
	}

	for (i = 0; i < numChildren; i++) {
		if (childClasses[i] != UxC_XrtGaCenter &&
			childClasses[i] != UxC_XrtGaCounter &&
			childClasses[i] != UxC_XrtGaFrame &&
			childClasses[i] != UxC_XrtGaCircularScale &&
			childClasses[i] != UxC_XrtGaNeedle &&
			childClasses[i] != UxC_XrtGaRange &&
			childClasses[i] != UxC_XrtGaText &&
			childClasses[i] != UxC_XrtGaTick) {
			return("XrtGaCircularGauge can't accept this object as a child\n");
		}
	}
	return(NULL);
}

/* Function that defines the XrtGaCircularGauge class */
void
UxRegister_XrtGaCircularGauge()
{
	int		n;
	
	xrt_debug_enter("UxRegister_XrtGaCircularGauge");

	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_XrtGaCircularGauge = UxRegister_class("XrtGaCircularGauge",
									UxC_primitive,
									sizeof(UxXrtGaCircularGauge),
									sizeof(UxXrtGaCircularGaugeClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtGaCircularGauge, "UxXrtGaCircularGauge.h");
	UxPutToolKitClass(UxC_XrtGaCircularGauge, (char *) xmXrtGaCircularGaugeWidgetClass);
	UxPutIconBitmap(UxC_XrtGaCircularGauge, "xrtGaCircularGauge.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtGaCircularGauge );
	
	for (n = 0; n < numXrtGaCircularGauge_res; n++) {
		if (XrtGaCircularGauge_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaCircularGauge_res_table, UxC_XrtGaCircularGauge);
		}
	}
	
	UxInit_method(UxC_XrtGaCircularGauge, UxM_UxWidgetCannotAcceptChildren, XrtGaCircularGaugeCanAddChild);

	xrt_debug_leave("UxRegister_XrtGaCircularGauge");
}

void
UxRegister_XrtGaCounter()
{
	int			n;
	
	xrt_debug_enter("UxRegister_XrtGaCounter");

	if (UxUndefined_class(UxC_gadget)) {
		UxRegister_gadget();
	}

	UxC_XrtGaCounter = UxRegister_class("XrtGaCounter",
									 UxC_gadget,
									 sizeof(UxXrtGaCounter),
									 sizeof(UxXrtGaCounterClass));

	/* General class initialization */
	UxPutUxFilename(UxC_XrtGaCounter, "UxXrtGaCounter.h" );
	UxPutToolKitClass(UxC_XrtGaCounter, (char *) xmXrtGaCounterObjectClass);
	UxPutIconBitmap(UxC_XrtGaCounter, "xrtGaCounter.xpm" );

	/* METHOD REGISTRATION */
	UxInit_method(UxC_XrtGaCounter, UxM_UxBuild, createUnmanaged);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_XrtGaCounter);

	presetGadgetResourcesOnXtObject(UxC_XrtGaCounter);

	for (n = 0; n < numXrtGaCounter_res; n++) {
		if (XrtGaCounter_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaCounter_res_table, UxC_XrtGaCounter);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaCounter");
}

static char* XrtGaLEDIconCanAddChild (parent, numChildren, childClasses, children)
swidget parent;
int numChildren;
Class_t *childClasses;
swidget *children;
{
	int	 	i;
	
	if (!UxIsSubclass(parent, UxC_XrtGaLEDIcon)) {
		return("Parent must be an XrtGaLEDIcon widget");
	}

	for (i = 0; i < numChildren; i++) {
		if (childClasses[i] != UxC_XrtGaIcon) {
			return("XrtGaLEDIcon can't accept this object as a child\n");
		}
	}
	return(NULL);
}

/* Function that defines the XrtGaLEDIcon class */
void
UxRegister_XrtGaLEDIcon()
{
	int		n;
	
	xrt_debug_enter("UxRegister_XrtGaLEDIcon");

	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_XrtGaLEDIcon = UxRegister_class("XrtGaLEDIcon",
									UxC_primitive,
									sizeof(UxXrtGaLEDIcon),
									sizeof(UxXrtGaLEDIconClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtGaLEDIcon, "UxXrtGaLEDIcon.h");
	UxPutToolKitClass(UxC_XrtGaLEDIcon, (char *) xmXrtGaLEDIconWidgetClass);
	UxPutIconBitmap(UxC_XrtGaLEDIcon, "xrtGaLEDIcon.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtGaLEDIcon );
	
	for (n = 0; n < numXrtGaLEDIcon_res; n++) {
		if (XrtGaLEDIcon_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaLEDIcon_res_table, UxC_XrtGaLEDIcon);
		}
	}
	
	UxInit_method(UxC_XrtGaLEDIcon, UxM_UxWidgetCannotAcceptChildren, XrtGaLEDIconCanAddChild);

	xrt_debug_leave("UxRegister_XrtGaLEDIcon");
}

/* Function that defines the XrtGaLEDNumerical class */
void
UxRegister_XrtGaLEDNumerical()
{
	int		n;
	
	xrt_debug_enter("UxRegister_XrtGaLEDNumerical");

	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_XrtGaLEDNumerical = UxRegister_class("XrtGaLEDNumerical",
									UxC_primitive,
									sizeof(UxXrtGaLEDNumerical),
									sizeof(UxXrtGaLEDNumericalClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtGaLEDNumerical, "UxXrtGaLEDNumerical.h");
	UxPutToolKitClass(UxC_XrtGaLEDNumerical, (char *) xmXrtGaLEDNumericalWidgetClass);
	UxPutIconBitmap(UxC_XrtGaLEDNumerical, "xrtGaLEDNumerical.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtGaLEDNumerical );
	
	for (n = 0; n < numXrtGaLEDNumerical_res; n++) {
		if (XrtGaLEDNumerical_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaLEDNumerical_res_table, UxC_XrtGaLEDNumerical);
			xrt_debug_print_int("LEDNum res:", n);
		}
	}
	
	xrt_debug_leave("UxRegister_XrtGaLEDNumerical");
}

static char* XrtGaLinearDialCanAddChild (parent, numChildren, childClasses, children)
swidget parent;
int numChildren;
Class_t *childClasses;
swidget *children;
{
	int	 	i;
	
	if (!UxIsSubclass(parent, UxC_XrtGaLinearDial)) {
		return("Parent must be an XrtGaLinearDial widget");
	}

	for (i = 0; i < numChildren; i++) {
		if (childClasses[i] != UxC_XrtGaFrame &&
			childClasses[i] != UxC_XrtGaLinearScale &&
			childClasses[i] != UxC_XrtGaIndicator &&
			childClasses[i] != UxC_XrtGaText &&
			childClasses[i] != UxC_XrtGaTick ) {
			return("XrtGaLinearDial can't accept this object as a child\n");
		}
	}
	return(NULL);
}

/* Function that defines the XrtGaLinearDial class */
void
UxRegister_XrtGaLinearDial()
{
	int		n;
	
	xrt_debug_enter("UxRegister_XrtGaLinearDial");

	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_XrtGaLinearDial = UxRegister_class("XrtGaLinearDial",
									UxC_primitive,
									sizeof(UxXrtGaLinearDial),
									sizeof(UxXrtGaLinearDialClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtGaLinearDial, "UxXrtGaLinearDial.h");
	UxPutToolKitClass(UxC_XrtGaLinearDial, (char *) xmXrtGaLinearDialWidgetClass);
	UxPutIconBitmap(UxC_XrtGaLinearDial, "xrtGaLinearDial.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtGaLinearDial );
	
	for (n = 0; n < numXrtGaLinearDial_res; n++) {
		if (XrtGaLinearDial_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaLinearDial_res_table, UxC_XrtGaLinearDial);
		}
	}
	
	UxInit_method(UxC_XrtGaLinearDial, UxM_UxWidgetCannotAcceptChildren, XrtGaLinearDialCanAddChild);

	xrt_debug_leave("UxRegister_XrtGaLinearDial");
}

static char* XrtGaLinearGaugeCanAddChild (parent, numChildren, childClasses, children)
swidget parent;
int numChildren;
Class_t *childClasses;
swidget *children;
{
	int	 	i;
	
	if (!UxIsSubclass(parent, UxC_XrtGaLinearGauge)) {
		return("Parent must be an XrtGaLinearGauge widget");
	}

	for (i = 0; i < numChildren; i++) {
		if (childClasses[i] != UxC_XrtGaFrame &&
			childClasses[i] != UxC_XrtGaLinearScale &&
			childClasses[i] != UxC_XrtGaIndicator &&
			childClasses[i] != UxC_XrtGaRange &&
			childClasses[i] != UxC_XrtGaText &&
			childClasses[i] != UxC_XrtGaTick ) {
			return("XrtGaLinearGauge can't accept this object as a child\n");
		}
	}
	return(NULL);
}

/* Function that defines the XrtGaLinearGauge class */
void
UxRegister_XrtGaLinearGauge()
{
	int		n;
	
	xrt_debug_enter("UxRegister_XrtGaLinearGauge");

	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_XrtGaLinearGauge = UxRegister_class("XrtGaLinearGauge",
									UxC_primitive,
									sizeof(UxXrtGaLinearGauge),
									sizeof(UxXrtGaLinearGaugeClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtGaLinearGauge, "UxXrtGaLinearGauge.h");
	UxPutToolKitClass(UxC_XrtGaLinearGauge, (char *) xmXrtGaLinearGaugeWidgetClass);
	UxPutIconBitmap(UxC_XrtGaLinearGauge, "xrtGaLinearGauge.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtGaLinearGauge );
	
	for (n = 0; n < numXrtGaLinearGauge_res; n++) {
		if (XrtGaLinearGauge_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaLinearGauge_res_table, UxC_XrtGaLinearGauge);
		}
	}
	
	UxInit_method(UxC_XrtGaLinearGauge, UxM_UxWidgetCannotAcceptChildren, XrtGaLinearGaugeCanAddChild);

	xrt_debug_leave("UxRegister_XrtGaLinearGauge");
}

static char* XrtGaPercentCanAddChild (parent, numChildren, childClasses, children)
swidget parent;
int numChildren;
Class_t *childClasses;
swidget *children;
{
	int	 	i;
	
	if (!UxIsSubclass(parent, UxC_XrtGaPercent)) {
		return("Parent must be an XrtGaPercent widget");
	}

	for (i = 0; i < numChildren; i++) {
		if (childClasses[i] != UxC_XrtGaFrame &&
			childClasses[i] != UxC_XrtGaItem &&
			childClasses[i] != UxC_XrtGaText &&
			childClasses[i] != UxC_XrtGaCircularScale &&
			childClasses[i] != UxC_XrtGaLinearScale) {
			return("XrtGaPercent can't accept this object as a child\n");
		}
	}
	return(NULL);
}

/* Function that defines the XrtGaPercent class */
void
UxRegister_XrtGaPercent()
{
	int		n;
	
	xrt_debug_enter("UxRegister_XrtGaPercent");

	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_XrtGaPercent = UxRegister_class("XrtGaPercent",
									UxC_primitive,
									sizeof(UxXrtGaPercent),
									sizeof(UxXrtGaPercentClass));

	/* General class initialization */

	UxPutUxFilename(UxC_XrtGaPercent, "UxXrtGaPercent.h");
	UxPutToolKitClass(UxC_XrtGaPercent, (char *) xmXrtGaPercentWidgetClass);
	UxPutIconBitmap(UxC_XrtGaPercent, "xrtGaPercent.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources( UxC_XrtGaPercent );
	
	for (n = 0; n < numXrtGaPercent_res; n++) {
		if (XrtGaPercent_res_table[n].used) {
			XrtGauge_define_resource(n, XrtGaPercent_res_table, UxC_XrtGaPercent);
		}
	}
	
	UxInit_method(UxC_XrtGaPercent, UxM_UxWidgetCannotAcceptChildren, XrtGaPercentCanAddChild);

	xrt_debug_leave("UxRegister_XrtGaPercent");
}

/*
 * To create XtObject in UIM/X we have to subclass it from XmGadget.
 * To do this properly we have to predefine (and set read-only) some
 * resources available for XmGadgets but not for XtObjects
 */

static void presetGadgetResourcesOnXtObject(class)
Class_t class;
{
  Resource_t	*res;
  
  res = UxGetRD_userData(class);
  res->Editable = False;
  res = UxGetRD_unitType(class);
  res->Editable = False;
  res = UxGetRD_traversalOn(class);
  res->Editable = False;
  res = UxGetRD_shadowThickness(class);
  res->Editable = False;
  res = UxGetRD_navigationType(class);
  res->Editable = False;
  res = UxGetRD_highlightThickness(class);
  res->Editable = False;
  res = UxGetRD_highlightOnEnter(class);
  res->Editable = False;
  res = UxGetRD_helpCallback(class);
  res->Editable = False;

  res = UxDefineResource(RD_EXAMPLE, UxGetRD_createManaged(class),
						 RD_EDITABLE, FALSE,
						 RD_END);
  UxPutClassResource(class, UxP_ShadowWidgetRD_createManaged, res);
}

static void createUnmanaged(newSwidget, manage)
swidget  newSwidget;
Boolean manage;
{
	/* Put the createManaged value will want. */
  UxPutInitialString(newSwidget, UxGetRD_createManaged(newSwidget), "\"false\"");

	/* The Superclass method MUST be called! */
  (void) UxVoid_get_op(UxC_gadget, UxM_UxBuild) (newSwidget, False);
}



