/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/gear widget UIM/X integration file
 *
 * $AstecId: XrtGear_rtime.c,v 1.3 1999/10/13 11:47:31 nakayama Exp $
 *
 *  SCCSID -- @(#)XrtGear_rtime.c	1.12    98/11/05 KL Group Inc.
 */

/******************************************************************************
				XrtGear_rtime.c
******************************************************************************/

#include <X11/Intrinsic.h>
#include "utype.h"

#include	<Xm/XrtGear.h>
#include	"XrtGear_xtype.h"
#include	"Xrt_restable.h"

static Boolean IsConverterInstalled();

/****
 ****   The format is to call UxrDDInstall to add a new converter.
 ****
 ****   UxrDDInstall( SquareNmajorDimension, UxUT_string, UxXT_MajorDimension );
 ****   UxrDDInstall( SquareNmakeSquare, UxUT_string, UxXt_Boolean );
 ****/


static void
XrtGear_install_converters(table, nres)
XrtGearResourceDesc	*table;
int			nres;
{
	int n;
	XrtGearResDataType type;
	
	for (n = 0; n < nres; n++) {
		if (table[n].used &&
			table[n].type->convert_flag == 1 &&
			IsConverterInstalled(table[n].xt_name) == False ) {
			UxDDInstall(table[n].xt_name,
						table[n].type->utype,
						table[n].type->xtype);
		}
	}
}

void
XrtGear_rtime()
{
	XrtGear_init_resource_table();
	XrtGear_install_converters(xrtAlnr_res_table, numAlnr_res);
	XrtGear_install_converters(xrtGLbl_res_table, numGLbl_res);
	XrtGear_install_converters(xrtPB_res_table,   numPB_res);
	XrtGear_install_converters(xrtTabB_res_table, numTabB_res);
	XrtGear_install_converters(xrtTabM_res_table, numTabM_res);
	XrtGear_install_converters(xrtTogB_res_table, numTogB_res);
	XrtGear_install_converters(xrtTbar_res_table, numTbar_res);	
	XrtGear_install_converters(xrtOutliner_res_table,   numOutliner_res);
	XrtGear_install_converters(xrtNodeStyle_res_table,  numNodeStyle_res);
	XrtGear_install_converters(xrtColumn_res_table,    	numColumn_res);
	XrtGear_install_converters(xrtGearCombo_res_table,  numGearCombo_res);
	XrtGear_install_converters(xrtProgress_res_table,   numProgress_res);
	XrtGear_install_converters(xrtPanedWindow_res_table,  numPanedWindow_res);

}


	/*
	 * Go through the list of multiply defined resources (lookupList)
	 * to see if converter for this type was installed.
	 * - If it is on the list and
	 *   - was not installed: install it, set the install flag to TRUE
	 *   - was installed: don't install again
	 * - If it isn't on the list : install it
	 */

static Boolean
IsConverterInstalled(xt_name)
char *xt_name;
{
	struct lookup {
		char 	*name;
		Boolean	installed;
	};	
	static struct lookup lookupList[] = {
		{XmNxrtGearAlignment, FALSE},
		{XmNxrtGearBorderType, FALSE}, 
		{XmNxrtGearDoubleBuffer, FALSE},
		{XmNxrtGearFontListSelected, FALSE}, 
		{XmNxrtGearLabelAlignment, FALSE},
		{XmNxrtGearLabelPixmap, FALSE},
		{XmNxrtGearLabelType, FALSE},
		{XmNxrtGearMask, FALSE},
		{XmNxrtGearOrientation, FALSE},
		{XmNxrtGearPixmapRotation, FALSE},
		{XmNxrtGearSpacing, FALSE},
		{XmNxrtGearStringLayout, FALSE},
		{XmNxrtGearStringRotation, FALSE},
		{XmNxrtGearWidthResizePolicy, FALSE}, 
		{XmNxrtGearWidthSizingPolicy, FALSE}, 
		{NULL, FALSE}
	};
  
	int i = 0;

	/* This are the Motif resources	we overwrite, (in Gear2.0) but we
	 * shouldn't try to install converters for them (they are already
	 * installed)
	 * 
	 * XmNbackground, XmNblinkRate, XmNcolumns, XmNcursorPosition,
	 * XmNcursorPositionVisible, XmNeditable, XmNfontList, XmNforeground,
	 * XmNhighlightThickness, XmNmaxLength, XmNmaximum, XmNmarginHeight,
	 * XmNmarginWidth, XmNminimum, XmNpendingDelete, XmNrecomputeSize,
	 * XmNresizeWidth, XmNselectThreshold, XmNscrollBarPlacement, XmNvalue,
	 * XmNverifyBell,
	 *
	 * If resource name does not begin with XrtGear then it is not Gear
	 * resource and we assume converter was insatleed
	 */

	if (strncmp(xt_name, "xrtGear", 7) ) {
	  return TRUE;
	}
		  
	/* This resources give "multiple defined" warning when Ux
	   code is generated, so we shouldn't install converters for them */
	
	for( i=0; lookupList[i].name != NULL; i++) {
		if( !strcmp( lookupList[i].name, xt_name )){
			if(lookupList[i].installed == FALSE) {
			  /* this type is on the list and has not been installed yet */
				lookupList[i].installed = TRUE;
				return FALSE;
			}
			/* this type has been installed */
			return TRUE;
		}
	}

	/* got here -> XmN(...) type was not on lookupList */
	return FALSE;
}


/*
 * UxConvertXrtGearIcon provides the way of converting resources of type
 * XrtGearIcon for UIM/X generated Xt code .
 * This function eventualy will be implemented in XRT/gear library and
 * there will be no need to do this hack.
 */

extern Widget UxTopLevel;

XrtGearIcon *   UxConvertXrtGearIcon( string )
char	*string;
{
	XrmValue	from, to;
	
	from.size = strlen( string ) + 1;
	from.addr = string;

	to.size = sizeof(XrtGearIcon);
	to.addr = (caddr_t)  XtNew(XrtGearIcon);

	((XrtGearIcon *)to.addr)->icon      = XmUNSPECIFIED_PIXMAP;
	((XrtGearIcon *)to.addr)->icon_mask = XmUNSPECIFIED_PIXMAP;
		
	if ( ! XtConvertAndStore( UxTopLevel, XmRString, &from,
						   XmRXrtGearIcon, &to )) {
	  printf("String -> XrtGearIcon conversion failed \n");
	}
	return ( (XrtGearIcon *)to.addr );	
}

